#! /usr/local/bin/scotty -inf
##
## Simple SNMP MIB browser for tkined.
##
## Copyright (c) 1993, 1994
##
## J. Schoenwaelder
## TU Braunschweig, Germany
## Institute for Operating Systems and Computer Networks
##
## Permission to use, copy, modify, and distribute this
## software and its documentation for any purpose and without
## fee is hereby granted, provided that this copyright
## notice appears in all copies.  The University of Braunschweig
## makes no representations about the suitability of this
## software for any purpose.  It is provided "as is" without
## express or implied warranty.
##

LoadDefaults snmp

SnmpInit SNMP-Browser

##
## Callback for walk button.
##

proc cb_walk { browser prefix } {
    foreach id [ined -noupdate select] {
	set comp [ined -noupdate retrieve $id]
	if {[ined type $comp] == "NODE"} {
	    set host [lindex [ined name $comp] 0]
	    set ip [GetIpAddress $comp]
	    if {$ip == ""} {
		ined acknowledge "Can not lookup IP Address for $host."
		return
	    }
	    SnmpWalk $ip $prefix
	}
    }
}

##
## Callback for the monitor button.
##

proc cb_monitor { browser prefix } {

    static monitor

    foreach id [ined -noupdate select] {
	lappend list [ined -noupdate retrieve $id]
    }

    if {![info exists list]} return

    if {![info exists monitor]} {
	set monitor [ined create INTERPRETER snmp_monitor.tcl]
    } else {
	if {[ined retrieve $monitor] == ""} {
	    set monitor [ined create INTERPRETER snmp_monitor.tcl]
	}
    }

    ined send $monitor monitor_variable $list $prefix
}

##
## Callback for show button (everything not in the first line).
##

proc cb_showit { browser prefix } {

    global snmp_verbose
    
    set subs [mib_ls $prefix]
    
    ined -noupdate clear $browser
    ined -noupdate hyperlink $browser "cb_walk $browser $prefix" --walk--
    ined -noupdate append $browser "\t"

    set last [split $prefix .]
    set this [lindex $last [expr [llength $last]-1]]
    set newlength [expr [llength $last]-2]
    set last [join [lrange $last 0 $newlength] .]
    if {$last != ""} {
	ined -noupdate hyperlink $browser "cb_showit $browser $last" ---up---
	ined -noupdate append $browser "\t"
    }

    set brothers [mib_ls $last]

    set idx [lsearch $brothers $this]
    incr idx -1
    set previous [lindex $brothers $idx]
    incr idx 2
    set next [lindex $brothers $idx]
    if {$previous != ""} {
	ined -noupdate hyperlink $browser \
	    "cb_showit $browser $last.$previous" -previous-
	ined -noupdate append $browser "\t"
    } else {
	ined -noupdate append $browser "\t\t"
    }
    if {$next != ""} {
	ined -noupdate hyperlink $browser \
	    "cb_showit $browser $last.$next" --next--
	ined -noupdate append $browser "\t"
    }

    if {$subs == ""} {
	set oid [mib parse_oid $prefix]
	set type [string toupper [mib type $oid]]
	if {[lsearch "GAUGE COUNTER INTEGER" $type] >= 0} {
	    ined -noupdate hyperlink $browser \
		"cb_monitor $browser $prefix" -monitor-
	}
    }

    ined -noupdate append $browser "\n\n"
    ined -noupdate append $browser "Current path: $prefix\n\n"

    if {$subs == ""} {
        set oid [mib parse_oid $prefix]
	if {$snmp_verbose == "true"} {
	    set desc [mib desc $oid]
	    regsub -all "\n" $desc "\n\t\t" desc
	    ined -noupdate append $browser "\t\t$desc"
	}
	cb_walk $browser $prefix
    } else {
	set toggle 0
	foreach elem $subs {
	    ined -noupdate hyperlink \
		$browser "cb_showit $browser $prefix.$elem" $elem
	    set len [string length $elem]
	    incr len +2
	    if {$toggle} { 
		set toggle 0
		ined -noupdate append $browser "\n" 
	    } else {
		set toggle 1
		if {$len < 8} {
		    ined -noupdate append $browser "\t\t\t\t"
		} elseif {$len < 16} {
		    ined -noupdate append $browser "\t\t\t"
		} elseif {$len < 24} {
		    ined -noupdate append $browser "\t\t"
		} else {
		    ined -noupdate append $browser "\t"
		}
	    }
	}
    }
}

##
## Get a listing of the known objects by a given prefix. This is
## indeed bad code since we scan all subidentifier fro 1 to 500.
## This may need some fixing sometimes.
##

proc mib_ls {prefix} { 

    global mib_dbm
    static cache	

    if {[info exists cache($prefix)]} {
	return $cache($prefix)
    }

    set nprefix [mib_find $prefix]
    
    set resultlist ""
    for {set i 1} {$i <= 500} {incr i} {
	if [dbm fetch $mib_dbm "n $nprefix.$i" data] {
	    set oid_split [split [lindex $data 0] .]
	    set uid [lindex $oid_split [expr [llength $oid_split]-1]]
	    set resultlist [concat $resultlist $uid]
	}
    }
    set cache($prefix) $resultlist

    return $resultlist
}

##
## Start a MIB browser for this device.
##

proc "MIB Browser" { list } {

    global snmp_browser
    
    set browser [ined -noupdate create LOG]
    lappend snmp_browser $browser
    ined -noupdate name $browser "SNMP MIB Browser"
    cb_showit $browser .iso.org.dod.internet.mgmt.mib-2
}

##
## Set the parameters (community, timeout, retry) for snmp requests.
##

proc "Set Parameter" {list} {
    SnmpParameter
}

##
## Show the defaults as loaded from the tkined.defaults files.
##

proc "Show Defaults" {list} {
    ShowDefaults
}

##
## Display some help about this tool.
##

proc "Help SNMP-Browser" {list} {
    ined browse "Help about SNMP-Browser" {
	"MIB Browser:" 
	"    Open a new window with a MIB browser. You can step through" 
	"    the MIB similar to a file selector. When reaching a leaf," 
	"    the selected nodes are queried for the selected object and" 
	"    a description of the objects will be displayed." 
	"" 
	"Set Parameter:" 
	"    This dialog allows you to set SNMP parameters like retries, " 
	"    timeouts, community name and port number. " 
	"" 
	"Show Defaults:" 
	"    Show the defaults that may be defined in the tkined.defaults" 
	"    files. This script makes use of definitions in the form" 
	"" 
	"        snmp.community:      <string>" 
	"        snmp.port:           <number>" 
	"        snmp.retries:        <number>" 
	"        snmp.timeout:        <number>" 
	"" 
	"        snmp.community.<ip>: <string>" 
	"        snmp.port.<ip>:      <number>" 
	"        snmp.retries.<ip>:   <number>" 
	"        snmp.timeout.<ip>:   <number>" 
	"" 
	"    where <ip> is an IP address in dot notation." 
    }
}

##
## Delete the menus created by this interpreter.
##

proc "Delete SNMP-Browser" {list} {
    global menus snmp_browser
    foreach id $snmp_browser { ined -noupdate delete $id }
    foreach id $menus { ined delete $id }
    exit
}

set menus [ ined create MENU "SNMP-Browser" \
	    "MIB Browser" "" \
	    "Set Parameter" "Show Defaults" "" \
	    "Help SNMP-Browser" "Delete SNMP-Browser" ]

